<?php

class ActiveRecord extends CActiveRecord
{
         /**
	 * Requires >= Yii 1.1.14;
	 *
	 * @recordAttributes is an array of key=>value pairs which can be inserted into @tableName
	 * @tableName is the table into which these values will be inserted.  Will attempt to determine the name itself
	 * 		if left to its own devices.
	 **/
	public function insertMany(array $recordAttributes, $tableName=null)
	{
		if( empty($tableName) )
			$tableName = $this->tableName();
		if( empty($recordAttributes) )
			return;

		$schema = Yii::app()->db->schema->getTable($tableName);
		
		// explicitly exclude the primary key so that we will not have an insert error
		// (otherwise it always tries to insert the pk value and it results in an error).
		if( isset($schema->columns) )
		{
			// get a copy of this ColumnSchema so we can add it back after
			// (we remove this so that it does not attempt to insert a primary key value)
			if( !is_array($schema->primaryKey) )
				$pkColumnSchema = $schema->columns[$schema->primaryKey];
			
			if( !is_array($schema->primaryKey) && isset($schema->columns[$schema->primaryKey]) )
				unset($schema->columns[$schema->primaryKey]);
		}
		$ret = $this->insertManyInChunks($recordAttributes, $schema);

		// re-apply the ColumnSchema for the primary key so that it won't fail on things such as $model->deleteByPk();
		if( !is_array($schema->primaryKey) && !empty($pkColumnSchema) && empty($schema->columns[$schema->primaryKey]) )
			$schema->columns[$schema->primaryKey] = $pkColumnSchema;

		return $ret;
	}

	/**
	 * Workhorse function for insertMany() so that large inserts are automatically broken into
	 * bite-sized chunks for the database.  Large sets could fail, so let's make it assume
	 * a large set and send off nibbles; we'll be better off for it.
	 *
	 * @param array $recordAttributes, arrays of key=>value pairs to be inserted.
	 * @param CDbTableSchema $tableSchema, required to insert data.
	 * @param int $chucnkSize, size of groupings for mass insertion.
	 *
	 * @returns int, the number of records inserted.
	 */
	private function insertManyInChunks(array $recordAttributes, $tableSchema, $chunkSize=300)
	{
		$ret = 0;
		$builder = Yii::app()->db->schema->commandBuilder;
		
		foreach( array_chunk($recordAttributes, $chunkSize) as $chunk )
		{
			$command = $builder->createMultipleInsertCommand($tableSchema, $chunk);
			$ret += $command->execute();
		}
		return $ret;
	}
}
?>